@@ -12,9 +12,9 @@ import math
 from collections import deque
 
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.5'
+__version__ = '0.6'
 __website__ = 'https://github.com/stefankoegl/kdtree'
-__license__ = 'GNU General Public License v3 or later'
+__license__ = 'ISC license'
 
 
 class Node(object):
@@ -272,79 +272,83 @@ class KDNode(Node):
         return (child, parent if parent is not None else self)
 
 
+    def should_remove(self, point, node):
+        """ checks if self's point (and maybe identity) matches """
+        if not self.data == point:
+            return False
+
+        return (node is None) or (node is self)
+
 
     @require_axis
-    def remove(self, point):
+    def remove(self, point, node=None):
         """ Removes the node with the given point from the tree
 
-        Returns the new root node of the (sub)tree """
+        Returns the new root node of the (sub)tree.
+
+        If there are multiple points matching "point", only one is removed. The
+        optional "node" parameter is used for checking the identity, once the
+        removeal candidate is decided."""
 
         # Recursion has reached an empty leaf node, nothing here to delete
         if not self:
             return
 
-
         # Recursion has reached the node to be deleted
-        if self.data == point:
-
-            if self.is_leaf:
-                self.data = None
-                return self
-
-            else:
-                # find a replacement for the node (will be the new subtree-root)
-                root, max_p = self.find_replacement()
-
-                pos = max_p.get_child_pos(root)
-
-                # self and root swap positions
-                tmp_l, tmp_r = self.left, self.right
-                self.left, self.right = root.left, root.right
-                root.left, root.right = tmp_l if tmp_l is not root else self, tmp_r if tmp_r is not root else self
-                self.axis, root.axis = root.axis, self.axis
-
-
-                # Special-case if we have chosen a direct child as the replacement
-                if max_p is not self:
-                    max_p.set_child(pos, self)
-                    new_depth = max_p.height()
-                    max_p.remove(self.data)
-
-                else:
-                    root.remove(self.data)
-
-                return root
-
+        if self.should_remove(point, node):
+            return self._remove(point)
 
         # Remove direct subnode
-        if self.left and self.left.data == point:
-            if self.left.is_leaf:
-                self.left = None
-
-            else:
-                self.left = self.left.remove(point)
-
-
-        elif self.right and self.right.data == point:
-            if self.right.is_leaf:
-                self.right = None
-
-            else:
-                self.right = self.right.remove(point)
+        if self.left and self.left.should_remove(point, node):
+            self.left = self.left._remove(point)
 
+        elif self.right and self.right.should_remove(point, node):
+            self.right = self.right._remove(point)
 
         # Recurse to subtrees
         if point[self.axis] <= self.data[self.axis]:
             if self.left:
-                self.left = self.left.remove(point)
+                self.left = self.left.remove(point, node)
 
         if point[self.axis] >= self.data[self.axis]:
             if self.right:
-                self.right = self.right.remove(point)
+                self.right = self.right.remove(point, node)
 
         return self
 
 
+    @require_axis
+    def _remove(self, point):
+        # we have reached the node to be deleted here
+
+        # deleting a leaf node is trivial
+        if self.is_leaf:
+            self.data = None
+            return self
+
+        # we have to delete a non-leaf node here
+
+        # find a replacement for the node (will be the new subtree-root)
+        root, max_p = self.find_replacement()
+
+        # self and root swap positions
+        tmp_l, tmp_r = self.left, self.right
+        self.left, self.right = root.left, root.right
+        root.left, root.right = tmp_l if tmp_l is not root else self, tmp_r if tmp_r is not root else self
+        self.axis, root.axis = root.axis, self.axis
+
+        # Special-case if we have not chosen a direct child as the replacement
+        if max_p is not self:
+            pos = max_p.get_child_pos(root)
+            max_p.set_child(pos, self)
+            max_p.remove(point, self)
+
+        else:
+            root.remove(point, self)
+
+        return root
+
+
     @property
     def is_balanced(self):
         """ Returns True if the (sub)tree is balanced
