@@ -21,6 +21,28 @@ except ImportError:
 class RemoveTest(unittest.TestCase):
 
 
+    def test_remove_duplicates(self):
+        """ creates a tree with only duplicate points, and removes them all """
+
+        points = [(1,1)] * 100
+        tree = kdtree.create(points)
+        self.assertTrue(tree.is_valid())
+
+        random.shuffle(points)
+        while points:
+            point = points.pop(0)
+
+            tree = tree.remove(point)
+
+            # Check if the Tree is valid after the removal
+            self.assertTrue(tree.is_valid())
+
+            # Check if the removal reduced the number of nodes by 1 (not more, not less)
+            remaining_points = len(points)
+            nodes_in_tree = len(list(tree.inorder()))
+            self.assertEqual(nodes_in_tree, remaining_points)
+
+
     def test_remove(self, num=100):
         """ Tests random removal from a tree, multiple times """
 
@@ -157,15 +179,15 @@ class NearestNeighbor(unittest.TestCase):
     def test_search_nn_dist(self):
         """ tests search_nn_dist() according to bug #8 """
 
-        points = [(x,y) for x in xrange(10) for y in xrange(10)]
+        points = [(x,y) for x in range(10) for y in range(10)]
         tree = kdtree.create(points)
         nn = tree.search_nn_dist((5,5), 2.5)
 
         self.assertEquals(len(nn), 4)
-        self.assertIn( (6,6), nn)
-        self.assertIn( (5,5), nn)
-        self.assertIn( (5,6), nn)
-        self.assertIn( (6,5), nn)
+        self.assertTrue( (6,6) in nn)
+        self.assertTrue( (5,5) in nn)
+        self.assertTrue( (5,6) in nn)
+        self.assertTrue( (6,5) in nn)
 
 
 def random_tree(nodes=20, dimensions=3, minval=0, maxval=100):
